@@ -3,9 +3,9 @@ import logging
 import re
 
 from sqlalchemy import event
+from sqlalchemy import inspect
 from sqlalchemy import schema as sa_schema
 from sqlalchemy import types as sqltypes
-from sqlalchemy.engine.reflection import Inspector
 from sqlalchemy.util import OrderedSet
 
 from alembic.ddl.base import _fk_spec
@@ -34,7 +34,7 @@ def _produce_net_changes(autogen_context, upgrade_ops):
     connection = autogen_context.connection
     include_schemas = autogen_context.opts.get("include_schemas", False)
 
-    inspector = Inspector.from_engine(connection)
+    inspector = inspect(connection)
 
     default_schema = connection.dialect.default_schema_name
     if include_schemas:
@@ -338,7 +338,9 @@ def _compare_columns(
 
 class _constraint_sig(object):
     def md_name_to_sql_name(self, context):
-        return self.name
+        return sqla_compat._get_constraint_final_name(
+            self.const, context.dialect
+        )
 
     def __eq__(self, other):
         return self.const == other.const
@@ -374,7 +376,9 @@ class _ix_constraint_sig(_constraint_sig):
         self.is_unique = bool(const.unique)
 
     def md_name_to_sql_name(self, context):
-        return sqla_compat._get_index_final_name(context.dialect, self.const)
+        return sqla_compat._get_constraint_final_name(
+            self.const, context.dialect
+        )
 
     @property
     def column_names(self):
@@ -497,6 +501,7 @@ def _compare_indexes_and_uniques(
             conn_indexes,
             metadata_unique_constraints,
             metadata_indexes,
+            autogen_context.dialect,
         )
 
     # 3. give the dialect a chance to omit indexes and constraints that
@@ -529,12 +534,12 @@ def _compare_indexes_and_uniques(
     metadata_names = dict(
         (c.md_name_to_sql_name(autogen_context), c)
         for c in metadata_unique_constraints.union(metadata_indexes)
-        if c.name is not None
+        if isinstance(c, _ix_constraint_sig)
+        or sqla_compat._constraint_is_named(c.const, autogen_context.dialect)
     )
 
     conn_uniques_by_name = dict((c.name, c) for c in conn_unique_constraints)
     conn_indexes_by_name = dict((c.name, c) for c in conn_indexes)
-
     conn_names = dict(
         (c.name, c)
         for c in conn_unique_constraints.union(conn_indexes)
@@ -556,7 +561,11 @@ def _compare_indexes_and_uniques(
     )
     metadata_indexes_by_sig = dict((ix.sig, ix) for ix in metadata_indexes)
     unnamed_metadata_uniques = dict(
-        (uq.sig, uq) for uq in metadata_unique_constraints if uq.name is None
+        (uq.sig, uq)
+        for uq in metadata_unique_constraints
+        if not sqla_compat._constraint_is_named(
+            uq.const, autogen_context.dialect
+        )
     )
 
     # assumptions:
@@ -719,6 +728,7 @@ def _correct_for_uq_duplicates_uix(
     conn_indexes,
     metadata_unique_constraints,
     metadata_indexes,
+    dialect,
 ):
     # dedupe unique indexes vs. constraints, since MySQL / Oracle
     # doesn't really have unique constraints as a separate construct.
@@ -726,25 +736,38 @@ def _correct_for_uq_duplicates_uix(
     # that already seem to be defined one way or the other
     # on that side.  This logic was formerly local to MySQL dialect,
     # generalized to Oracle and others. See #276
+
+    # resolve final rendered name for unique constraints defined in the
+    # metadata.   this includes truncation of long names.  naming convention
+    # names currently should already be set as cons.name, however leave this
+    # to the sqla_compat to decide.
+    metadata_cons_names = [
+        (sqla_compat._get_constraint_final_name(cons, dialect), cons)
+        for cons in metadata_unique_constraints
+    ]
+
     metadata_uq_names = set(
-        [
-            cons.name
-            for cons in metadata_unique_constraints
-            if cons.name is not None
-        ]
+        name for name, cons in metadata_cons_names if name is not None
     )
 
     unnamed_metadata_uqs = set(
         [
             _uq_constraint_sig(cons).sig
-            for cons in metadata_unique_constraints
-            if cons.name is None
+            for name, cons in metadata_cons_names
+            if name is None
         ]
     )
 
     metadata_ix_names = set(
-        [cons.name for cons in metadata_indexes if cons.unique]
+        [
+            sqla_compat._get_constraint_final_name(cons, dialect)
+            for cons in metadata_indexes
+            if cons.unique
+        ]
     )
+
+    # for reflection side, names are in their final database form
+    # already since they're from the database
     conn_ix_names = dict(
         (cons.name, cons) for cons in conn_indexes if cons.unique
     )
@@ -809,7 +832,7 @@ def _setup_autoincrement(
 
     if metadata_col.table._autoincrement_column is metadata_col:
         alter_column_op.kw["autoincrement"] = True
-    elif util.sqla_110 and metadata_col.autoincrement is True:
+    elif metadata_col.autoincrement is True:
         alter_column_op.kw["autoincrement"] = True
     elif metadata_col.autoincrement is False:
         alter_column_op.kw["autoincrement"] = False
@@ -903,6 +926,18 @@ def _compare_server_default(
     conn_col_default = conn_col.server_default
     if conn_col_default is None and metadata_default is None:
         return False
+
+    if sqla_compat.has_computed and isinstance(
+        metadata_default, sa_schema.Computed
+    ):
+        # return False in case of a computed column as the server
+        # default. Note that DDL for adding or removing "GENERATED AS" from
+        # an existing column is not currently known for any backend.
+        # Once SQLAlchemy can reflect "GENERATED" as the "computed" element,
+        # we would also want to ignore and/or warn for changes vs. the
+        # metadata (or support backend specific DDL if applicable).
+        return False
+
     rendered_metadata_default = _render_server_default_for_compare(
         metadata_default, metadata_col, autogen_context
     )
