@@ -4,6 +4,10 @@ from alembic.util import sqla_compat
 
 
 class DefaultRequirements(SuiteRequirements):
+    @property
+    def unicode_string(self):
+        return exclusions.skip_if(["oracle"])
+
     @property
     def alter_column(self):
         return exclusions.skip_if(["sqlite"], "no ALTER COLUMN support")
@@ -86,6 +90,12 @@ class DefaultRequirements(SuiteRequirements):
             "postgresql", "oracle", "mssql", "sybase", "sqlite"
         )
 
+    @property
+    def datetime_timezone(self):
+        """target dialect supports timezone with datetime types."""
+
+        return exclusions.only_on(["postgresql"])
+
     @property
     def postgresql_uuid_ossp(self):
         def check_uuid_ossp(config):
