@@ -11,13 +11,13 @@ from sqlalchemy import ForeignKey
 from sqlalchemy import ForeignKeyConstraint
 from sqlalchemy import func
 from sqlalchemy import Index
+from sqlalchemy import inspect
 from sqlalchemy import Integer
 from sqlalchemy import MetaData
 from sqlalchemy import PrimaryKeyConstraint
 from sqlalchemy import String
 from sqlalchemy import Table
 from sqlalchemy import UniqueConstraint
-from sqlalchemy.engine.reflection import Inspector
 from sqlalchemy.schema import CreateIndex
 from sqlalchemy.schema import CreateTable
 from sqlalchemy.sql import column
@@ -34,13 +34,15 @@ from alembic.testing import exclusions
 from alembic.testing import mock
 from alembic.testing import TestBase
 from alembic.testing.fixtures import op_fixture
+from alembic.util import exc as alembic_exc
+from alembic.util.sqla_compat import sqla_14
 
 
 class BatchApplyTest(TestBase):
     def setUp(self):
         self.op = Operations(mock.Mock(opts={}))
 
-    def _simple_fixture(self, table_args=(), table_kwargs={}):
+    def _simple_fixture(self, table_args=(), table_kwargs={}, **kw):
         m = MetaData()
         t = Table(
             "tname",
@@ -49,7 +51,7 @@ class BatchApplyTest(TestBase):
             Column("x", String(10)),
             Column("y", Integer),
         )
-        return ApplyBatchImpl(t, table_args, table_kwargs, False)
+        return ApplyBatchImpl(t, table_args, table_kwargs, False, **kw)
 
     def _uq_fixture(self, table_args=(), table_kwargs={}):
         m = MetaData()
@@ -270,11 +272,12 @@ class BatchApplyTest(TestBase):
         )
 
         args["tname_colnames"] = ", ".join(
-            "CAST(%(schema)stname.%(name)s AS %(type)s) AS anon_1"
+            "CAST(%(schema)stname.%(name)s AS %(type)s) AS %(cast_label)s"
             % {
                 "schema": args["schema"],
                 "name": name,
                 "type": impl.new_table.c[name].type,
+                "cast_label": name if sqla_14 else "anon_1",
             }
             if (
                 impl.new_table.c[name].type._type_affinity
@@ -464,6 +467,98 @@ class BatchApplyTest(TestBase):
         new_table = self._assert_impl(impl, colnames=["id", "x", "y", "g"])
         eq_(new_table.c.g.name, "g")
 
+    def test_partial_reordering(self):
+        impl = self._simple_fixture(partial_reordering=[("x", "id", "y")])
+        new_table = self._assert_impl(impl, colnames=["x", "id", "y"])
+        eq_(new_table.c.x.name, "x")
+
+    def test_add_col_partial_reordering(self):
+        impl = self._simple_fixture(partial_reordering=[("id", "x", "g", "y")])
+        col = Column("g", Integer)
+        # operations.add_column produces a table
+        t = self.op.schema_obj.table("tname", col)  # noqa
+        impl.add_column("tname", col)
+        new_table = self._assert_impl(impl, colnames=["id", "x", "g", "y"])
+        eq_(new_table.c.g.name, "g")
+
+    def test_add_col_insert_before(self):
+        impl = self._simple_fixture()
+        col = Column("g", Integer)
+        # operations.add_column produces a table
+        t = self.op.schema_obj.table("tname", col)  # noqa
+        impl.add_column("tname", col, insert_before="x")
+        new_table = self._assert_impl(impl, colnames=["id", "g", "x", "y"])
+        eq_(new_table.c.g.name, "g")
+
+    def test_add_col_insert_before_beginning(self):
+        impl = self._simple_fixture()
+        impl.add_column("tname", Column("g", Integer), insert_before="id")
+        new_table = self._assert_impl(impl, colnames=["g", "id", "x", "y"])
+        eq_(new_table.c.g.name, "g")
+
+    def test_add_col_insert_before_middle(self):
+        impl = self._simple_fixture()
+        impl.add_column("tname", Column("g", Integer), insert_before="y")
+        new_table = self._assert_impl(impl, colnames=["id", "x", "g", "y"])
+        eq_(new_table.c.g.name, "g")
+
+    def test_add_col_insert_after_middle(self):
+        impl = self._simple_fixture()
+        impl.add_column("tname", Column("g", Integer), insert_after="id")
+        new_table = self._assert_impl(impl, colnames=["id", "g", "x", "y"])
+        eq_(new_table.c.g.name, "g")
+
+    def test_add_col_insert_after_penultimate(self):
+        impl = self._simple_fixture()
+        impl.add_column("tname", Column("g", Integer), insert_after="x")
+        self._assert_impl(impl, colnames=["id", "x", "g", "y"])
+
+    def test_add_col_insert_after_end(self):
+        impl = self._simple_fixture()
+        impl.add_column("tname", Column("g", Integer), insert_after="y")
+        new_table = self._assert_impl(impl, colnames=["id", "x", "y", "g"])
+        eq_(new_table.c.g.name, "g")
+
+    def test_add_col_insert_after_plus_no_order(self):
+        impl = self._simple_fixture()
+        # operations.add_column produces a table
+        impl.add_column("tname", Column("g", Integer), insert_after="id")
+        impl.add_column("tname", Column("q", Integer))
+        new_table = self._assert_impl(
+            impl, colnames=["id", "g", "x", "y", "q"]
+        )
+        eq_(new_table.c.g.name, "g")
+
+    def test_add_col_no_order_plus_insert_after(self):
+        impl = self._simple_fixture()
+        col = Column("g", Integer)
+        # operations.add_column produces a table
+        t = self.op.schema_obj.table("tname", col)  # noqa
+        impl.add_column("tname", Column("q", Integer))
+        impl.add_column("tname", Column("g", Integer), insert_after="id")
+        new_table = self._assert_impl(
+            impl, colnames=["id", "g", "x", "y", "q"]
+        )
+        eq_(new_table.c.g.name, "g")
+
+    def test_add_col_insert_after_another_insert(self):
+        impl = self._simple_fixture()
+        impl.add_column("tname", Column("g", Integer), insert_after="id")
+        impl.add_column("tname", Column("q", Integer), insert_after="g")
+        new_table = self._assert_impl(
+            impl, colnames=["id", "g", "q", "x", "y"]
+        )
+        eq_(new_table.c.g.name, "g")
+
+    def test_add_col_insert_before_another_insert(self):
+        impl = self._simple_fixture()
+        impl.add_column("tname", Column("g", Integer), insert_after="id")
+        impl.add_column("tname", Column("q", Integer), insert_before="g")
+        new_table = self._assert_impl(
+            impl, colnames=["id", "q", "g", "x", "y"]
+        )
+        eq_(new_table.c.g.name, "g")
+
     def test_add_server_default(self):
         impl = self._simple_fixture()
         impl.alter_column("tname", "y", server_default="10")
@@ -899,7 +994,8 @@ class CopyFromTest(TestBase):
             "CREATE TABLE _alembic_tmp_foo (id INTEGER NOT NULL, "
             "data INTEGER, x INTEGER, PRIMARY KEY (id))",
             "INSERT INTO _alembic_tmp_foo (id, data, x) SELECT foo.id, "
-            "CAST(foo.data AS INTEGER) AS anon_1, foo.x FROM foo",
+            "CAST(foo.data AS INTEGER) AS %s, foo.x FROM foo"
+            % (("data" if sqla_14 else "anon_1"),),
             "DROP TABLE foo",
             "ALTER TABLE _alembic_tmp_foo RENAME TO foo",
         )
@@ -922,7 +1018,8 @@ class CopyFromTest(TestBase):
             "CREATE TABLE _alembic_tmp_foo (id INTEGER NOT NULL, "
             "data VARCHAR(50), x INTEGER, y INTEGER, PRIMARY KEY (id))",
             "INSERT INTO _alembic_tmp_foo (id, data, x, y) SELECT foo.id, "
-            "foo.data, foo.x, CAST(foo.y AS INTEGER) AS anon_1 FROM foo",
+            "foo.data, foo.x, CAST(foo.y AS INTEGER) AS %s FROM foo"
+            % (("y" if sqla_14 else "anon_1"),),
             "DROP TABLE foo",
             "ALTER TABLE _alembic_tmp_foo RENAME TO foo",
         )
@@ -944,7 +1041,8 @@ class CopyFromTest(TestBase):
             "data VARCHAR(50), x INTEGER, y BOOLEAN, PRIMARY KEY (id), "
             "CONSTRAINT ck1 CHECK (y IN (0, 1)))",
             "INSERT INTO _alembic_tmp_foo (id, data, x, y) SELECT foo.id, "
-            "foo.data, foo.x, CAST(foo.y AS BOOLEAN) AS anon_1 FROM foo",
+            "foo.data, foo.x, CAST(foo.y AS BOOLEAN) AS %s FROM foo"
+            % (("y" if sqla_14 else "anon_1"),),
             "DROP TABLE foo",
             "ALTER TABLE _alembic_tmp_foo RENAME TO foo",
         )
@@ -1015,7 +1113,8 @@ class CopyFromTest(TestBase):
             "CREATE TABLE _alembic_tmp_foo (id INTEGER NOT NULL, "
             "data INTEGER, x INTEGER, PRIMARY KEY (id))",
             "INSERT INTO _alembic_tmp_foo (id, data, x) SELECT foo.id, "
-            "CAST(foo.data AS INTEGER) AS anon_1, foo.x FROM foo",
+            "CAST(foo.data AS INTEGER) AS %s, foo.x FROM foo"
+            % (("data" if sqla_14 else "anon_1"),),
             "DROP TABLE foo",
             "ALTER TABLE _alembic_tmp_foo RENAME TO foo",
             "CREATE UNIQUE INDEX ix_data ON foo (data)",
@@ -1147,7 +1246,7 @@ class BatchRoundTripTest(TestBase):
                 type_=Integer,
                 existing_type=Boolean(create_constraint=True, name="ck1"),
             )
-        insp = Inspector.from_engine(config.db)
+        insp = inspect(config.db)
 
         eq_(
             [
@@ -1196,7 +1295,7 @@ class BatchRoundTripTest(TestBase):
         self._boolean_fixture()
         with self.op.batch_alter_table("hasbool") as batch_op:
             batch_op.drop_column("x")
-        insp = Inspector.from_engine(config.db)
+        insp = inspect(config.db)
 
         assert "x" not in (c["name"] for c in insp.get_columns("hasbool"))
 
@@ -1206,7 +1305,7 @@ class BatchRoundTripTest(TestBase):
             batch_op.alter_column(
                 "x", type_=Boolean(create_constraint=True, name="ck1")
             )
-        insp = Inspector.from_engine(config.db)
+        insp = inspect(config.db)
 
         if exclusions.against(config, "sqlite"):
             eq_(
@@ -1247,7 +1346,7 @@ class BatchRoundTripTest(TestBase):
             batch_op.alter_column("data", type_=String(30))
             batch_op.create_index("ix_data", ["data"])
 
-        insp = Inspector.from_engine(config.db)
+        insp = inspect(config.db)
         eq_(
             set(
                 (ix["name"], tuple(ix["column_names"]))
@@ -1291,7 +1390,7 @@ class BatchRoundTripTest(TestBase):
                 "data", new_column_name="newdata", existing_type=String(50)
             )
 
-        insp = Inspector.from_engine(self.conn)
+        insp = inspect(self.conn)
         eq_(
             [
                 (
@@ -1338,9 +1437,8 @@ class BatchRoundTripTest(TestBase):
                 "data", new_column_name="newdata", existing_type=String(50)
             )
 
-        insp = Inspector.from_engine(self.conn)
+        insp = inspect(self.conn)
 
-        insp = Inspector.from_engine(self.conn)
         eq_(
             [
                 (
@@ -1388,7 +1486,7 @@ class BatchRoundTripTest(TestBase):
             batch_op.drop_column("id")
             batch_op.add_column(Column("id", Integer))
 
-        pk_const = Inspector.from_engine(self.conn).get_pk_constraint("foo")
+        pk_const = inspect(self.conn).get_pk_constraint("foo")
         eq_(pk_const["constrained_columns"], [])
 
     def test_drop_pk_col_readd_pk_col(self):
@@ -1397,7 +1495,7 @@ class BatchRoundTripTest(TestBase):
             batch_op.drop_column("id")
             batch_op.add_column(Column("id", Integer, primary_key=True))
 
-        pk_const = Inspector.from_engine(self.conn).get_pk_constraint("foo")
+        pk_const = inspect(self.conn).get_pk_constraint("foo")
         eq_(pk_const["constrained_columns"], ["id"])
 
     def test_drop_pk_col_readd_col_also_pk_const(self):
@@ -1408,7 +1506,7 @@ class BatchRoundTripTest(TestBase):
             batch_op.add_column(Column("id", Integer))
             batch_op.create_primary_key("newpk", ["id"])
 
-        pk_const = Inspector.from_engine(self.conn).get_pk_constraint("foo")
+        pk_const = inspect(self.conn).get_pk_constraint("foo")
         eq_(pk_const["constrained_columns"], ["id"])
 
     def test_add_pk_constraint(self):
@@ -1416,7 +1514,7 @@ class BatchRoundTripTest(TestBase):
         with self.op.batch_alter_table("nopk", recreate="always") as batch_op:
             batch_op.create_primary_key("newpk", ["a", "b"])
 
-        pk_const = Inspector.from_engine(self.conn).get_pk_constraint("nopk")
+        pk_const = inspect(self.conn).get_pk_constraint("nopk")
         with config.requirements.reflects_pk_names.fail_if():
             eq_(pk_const["name"], "newpk")
         eq_(pk_const["constrained_columns"], ["a", "b"])
@@ -1438,7 +1536,6 @@ class BatchRoundTripTest(TestBase):
             {"id": 6, "data": 5, "x": -2},
         )
 
-    @config.requirements.sqlalchemy_094
     @config.requirements.unnamed_constraints
     def test_drop_foreign_key(self):
         bar = Table(
@@ -1458,7 +1555,7 @@ class BatchRoundTripTest(TestBase):
             "bar", naming_convention=naming_convention
         ) as batch_op:
             batch_op.drop_constraint("fk_bar_foo_id_foo", type_="foreignkey")
-        eq_(Inspector.from_engine(self.conn).get_foreign_keys("bar"), [])
+        eq_(inspect(self.conn).get_foreign_keys("bar"), [])
 
     def test_drop_column_fk_recreate(self):
         with self.op.batch_alter_table("foo", recreate="always") as batch_op:
@@ -1589,6 +1686,64 @@ class BatchRoundTripTest(TestBase):
                 {"id": 5, "data": "d5", "x": 9, "data2": "hi"},
             ]
         )
+        eq_(
+            [col["name"] for col in inspect(config.db).get_columns("foo")],
+            ["id", "data", "x", "data2"],
+        )
+
+    def test_add_column_insert_before_recreate(self):
+        with self.op.batch_alter_table("foo", recreate="always") as batch_op:
+            batch_op.add_column(
+                Column("data2", String(50), server_default="hi"),
+                insert_before="data",
+            )
+        self._assert_data(
+            [
+                {"id": 1, "data": "d1", "x": 5, "data2": "hi"},
+                {"id": 2, "data": "22", "x": 6, "data2": "hi"},
+                {"id": 3, "data": "8.5", "x": 7, "data2": "hi"},
+                {"id": 4, "data": "9.46", "x": 8, "data2": "hi"},
+                {"id": 5, "data": "d5", "x": 9, "data2": "hi"},
+            ]
+        )
+        eq_(
+            [col["name"] for col in inspect(config.db).get_columns("foo")],
+            ["id", "data2", "data", "x"],
+        )
+
+    def test_add_column_insert_after_recreate(self):
+        with self.op.batch_alter_table("foo", recreate="always") as batch_op:
+            batch_op.add_column(
+                Column("data2", String(50), server_default="hi"),
+                insert_after="data",
+            )
+        self._assert_data(
+            [
+                {"id": 1, "data": "d1", "x": 5, "data2": "hi"},
+                {"id": 2, "data": "22", "x": 6, "data2": "hi"},
+                {"id": 3, "data": "8.5", "x": 7, "data2": "hi"},
+                {"id": 4, "data": "9.46", "x": 8, "data2": "hi"},
+                {"id": 5, "data": "d5", "x": 9, "data2": "hi"},
+            ]
+        )
+        eq_(
+            [col["name"] for col in inspect(config.db).get_columns("foo")],
+            ["id", "data", "data2", "x"],
+        )
+
+    def test_add_column_insert_before_raise_on_alter(self):
+        def go():
+            with self.op.batch_alter_table("foo") as batch_op:
+                batch_op.add_column(
+                    Column("data2", String(50), server_default="hi"),
+                    insert_before="data",
+                )
+
+        assert_raises_message(
+            alembic_exc.CommandError,
+            "Can't specify insert_before or insert_after when using ALTER",
+            go,
+        )
 
     def test_add_column_recreate(self):
         with self.op.batch_alter_table("foo", recreate="always") as batch_op:
@@ -1605,9 +1760,13 @@ class BatchRoundTripTest(TestBase):
                 {"id": 5, "data": "d5", "x": 9, "data2": "hi"},
             ]
         )
+        eq_(
+            [col["name"] for col in inspect(config.db).get_columns("foo")],
+            ["id", "data", "x", "data2"],
+        )
 
     def test_create_drop_index(self):
-        insp = Inspector.from_engine(config.db)
+        insp = inspect(config.db)
         eq_(insp.get_indexes("foo"), [])
 
         with self.op.batch_alter_table("foo", recreate="always") as batch_op:
@@ -1623,7 +1782,7 @@ class BatchRoundTripTest(TestBase):
             ]
         )
 
-        insp = Inspector.from_engine(config.db)
+        insp = inspect(config.db)
         eq_(
             [
                 dict(
@@ -1639,7 +1798,7 @@ class BatchRoundTripTest(TestBase):
         with self.op.batch_alter_table("foo", recreate="always") as batch_op:
             batch_op.drop_index("ix_data")
 
-        insp = Inspector.from_engine(config.db)
+        insp = inspect(config.db)
         eq_(insp.get_indexes("foo"), [])
 
 
