@@ -2,12 +2,12 @@ from sqlalchemy import Boolean
 from sqlalchemy import Column
 from sqlalchemy import DATETIME
 from sqlalchemy import func
+from sqlalchemy import inspect
 from sqlalchemy import Integer
 from sqlalchemy import MetaData
 from sqlalchemy import Table
 from sqlalchemy import text
 from sqlalchemy import TIMESTAMP
-from sqlalchemy.engine.reflection import Inspector
 
 from alembic import op
 from alembic import util
@@ -19,6 +19,7 @@ from alembic.testing.env import staging_env
 from alembic.testing.fixtures import AlterColRoundTripFixture
 from alembic.testing.fixtures import op_fixture
 from alembic.testing.fixtures import TestBase
+from alembic.util import sqla_compat
 
 
 class MySQLOpTest(TestBase):
@@ -309,6 +310,23 @@ class MySQLOpTest(TestBase):
             "COMMENT 'existing column comment'"
         )
 
+    @config.requirements.comments_api
+    def test_rename_column_existing_comment(self):
+        context = op_fixture("mysql")
+        op.alter_column(
+            "t1",
+            "c1",
+            new_column_name="newc1",
+            existing_nullable=False,
+            existing_comment="existing column comment",
+            existing_type=Integer,
+        )
+
+        context.assert_(
+            "ALTER TABLE t1 CHANGE c1 newc1 INTEGER NOT NULL "
+            "COMMENT 'existing column comment'"
+        )
+
     @config.requirements.comments_api
     def test_alter_column_new_comment_replaces_existing(self):
         context = op_fixture("mysql")
@@ -341,6 +359,18 @@ class MySQLOpTest(TestBase):
         op.drop_table_comment("t2", existing_comment="t2 table", schema="foo")
         context.assert_("ALTER TABLE foo.t2 COMMENT ''")
 
+    @config.requirements.computed_columns_api
+    def test_add_column_computed(self):
+        context = op_fixture("mysql")
+        op.add_column(
+            "t1",
+            Column("some_column", Integer, sqla_compat.Computed("foo * 5")),
+        )
+        context.assert_(
+            "ALTER TABLE t1 ADD COLUMN some_column "
+            "INTEGER GENERATED ALWAYS AS (foo * 5)"
+        )
+
     def test_drop_fk(self):
         context = op_fixture("mysql")
         op.drop_constraint("f1", "t1", "foreignkey")
@@ -526,7 +556,7 @@ class MySQLDefaultCompareTest(TestBase):
 
     def _compare_default(self, t1, t2, col, rendered):
         t1.create(self.bind)
-        insp = Inspector.from_engine(self.bind)
+        insp = inspect(self.bind)
         cols = insp.get_columns(t1.name)
         refl = Table(t1.name, MetaData())
         insp.reflecttable(refl, None)
